'use strict';

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _getWindowSize = require('./get-window-size');

var _getWindowSize2 = _interopRequireDefault(_getWindowSize);

var _findWindow = require('./find-window');

var _findWindow2 = _interopRequireDefault(_findWindow);

var _utilsExec = require('../utils/exec');

var _binaries = require('../binaries');

var _binaries2 = _interopRequireDefault(_binaries);

function getCorrectedSize(currentClientAreaSize, currentWindowSize, requestedSize) {
    var horizontalChrome = currentWindowSize.width - currentClientAreaSize.width;
    var verticalChrome = currentWindowSize.height - currentClientAreaSize.height;

    return {
        width: requestedSize.width + horizontalChrome,
        height: requestedSize.height + verticalChrome
    };
}

/**
 * Changes the browser's client area size to the new width and height.
 * @function
 * @async
 * @name resize
 * @param {string} pageTitle - Specifies the title of the web page opened in the browser.
 * @param {number} currentWidth - Specifies the current width of the browser's client area, in pixels. Use the window.innerWidth property to determine it.
 * @param {number} currentHeight - Specifies the current height of the browser's client area, in pixels. Use the window.innerHeight property to determine it.
 * @param {number} width - Specifies the new client area width, in pixels.
 * @param {number} height - Specifies the new client area height, in pixels.
 **/

exports.default = function callee$0$0(pageTitle, currentWidth, currentHeight, width, height) {
    var windowDescription, resizeArguments, currentClientAreaSize, requestedSize, currentWindowSize, correctedSize;
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.next = 2;
                return _regeneratorRuntime.awrap(_findWindow2.default(pageTitle));

            case 2:
                windowDescription = context$1$0.sent;

                if (windowDescription) {
                    context$1$0.next = 5;
                    break;
                }

                return context$1$0.abrupt('return');

            case 5:
                resizeArguments = void 0;

                if (!_osFamily2.default.win) {
                    context$1$0.next = 10;
                    break;
                }

                resizeArguments = [windowDescription.hwnd];
                context$1$0.next = 19;
                break;

            case 10:
                if (!_osFamily2.default.mac) {
                    context$1$0.next = 14;
                    break;
                }

                resizeArguments = [windowDescription.windowId, windowDescription.bundleId];
                context$1$0.next = 19;
                break;

            case 14:
                if (!_osFamily2.default.linux) {
                    context$1$0.next = 18;
                    break;
                }

                resizeArguments = [windowDescription.windowId];
                context$1$0.next = 19;
                break;

            case 18:
                return context$1$0.abrupt('return');

            case 19:
                currentClientAreaSize = { width: currentWidth, height: currentHeight };
                requestedSize = { width: width, height: height };
                context$1$0.next = 23;
                return _regeneratorRuntime.awrap(_getWindowSize2.default(windowDescription));

            case 23:
                currentWindowSize = context$1$0.sent;

                if (currentWindowSize) {
                    context$1$0.next = 26;
                    break;
                }

                return context$1$0.abrupt('return');

            case 26:
                correctedSize = getCorrectedSize(currentClientAreaSize, currentWindowSize, requestedSize);

                resizeArguments = resizeArguments.concat([correctedSize.width, correctedSize.height]);

                context$1$0.next = 30;
                return _regeneratorRuntime.awrap(_utilsExec.execFile(_binaries2.default.resize, resizeArguments));

            case 30:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
};

module.exports = exports.default;